// file		: functions.cpp
// filetype     : jjs-bdd demo file
// authors	: Sascha Klueppelholz, Joern Ossowski
// date		: 02-11-2004
// comment	: demo of the basic functions and operators

#include <iostream>

// using bdd functions, only this include is needed
#include <sobdd/sobddfunction.h>

using namespace std;

int main(int argc, char *argv[]){ 
  cout << endl << "Starting jjs-bdd demo for function usage. Take a look at the source file for more details." << endl << endl;
  
  // adding a new function to your programm is very easy
  // uninitialized functions are error, which is stable under (boolean) operations
  SOBDDFunction anewfunction;
  cout << "... an uninitalized function equals SOBDDFunction::ERROR(): " << (anewfunction==SOBDDFunction::ERROR()) << endl;
  cout << "... and keeps error after any operation: " << (OR(SOBDDFunction::ONE(),anewfunction)==SOBDDFunction::ERROR()) << endl;

  // the constant one and zero functions are members of the package
  SOBDDFunction constantone  = SOBDDFunction::ONE();
  SOBDDFunction constantzero = SOBDDFunction::ZERO();

  // you can negate functions and check if they are equal
  cout << "... is NOT(ZERO)==ONE()?: " << (NOT(constantone)==constantzero) << endl;

  // the projection function to a specific variable can be added
  // if the new variable is not already in the variable order, it will be appended automaticly
  SOBDDFunction p_x=SOBDDFunction::projection("x");
  SOBDDFunction p_y=SOBDDFunction::projection("y");

  // not is not the only boolean operator to apply on functions
  SOBDDFunction test1,test2,test3;
  test1=AND(p_x,NOT(p_y));
  test2=OR(p_x,p_y);
  test3=NOT(XOR(test1,test2));

  // the most general way of appling operations is the ITE (if then else) operator
  // which represents the shannon decomposition in bdds
  cout << "... ITE(ONE(),f_x,f_y)==f_x)?: " <<  (ITE(SOBDDFunction::ONE(),p_x,p_y)==p_x) << endl;
  cout << "... ITE(ZERO(),f_x,f_y)==f_y?: " <<  (ITE(SOBDDFunction::ZERO(),p_x,p_y)==p_y) << endl;
  cout << "... ITE(p_x,NOT(f_y),f_y)==XOR(f_x,f_y)?: " << (ITE(p_x,NOT(p_y),p_y)==XOR(p_x,p_y)) << endl;
  	
  return EXIT_SUCCESS;
}
